/* util.c - auxiliary functions for the libgo library
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: util.c,v 1.1.1.1 2001/08/24 20:40:58 voss Exp $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

#include <glib.h>

#include "go.h"


int
str_to_int (const char *str, int def)
{
  char *tail;
  int  val;

  if (! str)  return def;
  errno = 0;
  val = strtol (str, &tail, 0);
  if (errno == ERANGE && *tail == '\0') {
    g_error ("parse error: invalid integer number \"%s\"", str);
  } else if (errno || tail == str || *tail != '\0') {
    g_error ("parse error: malformed integer number \"%s\"", str);
  }
  return  val;
}

double
str_to_double (const char *str, double def)
{
  char *tail;
  double  val;

  if (! str)  return def;
  errno = 0;
  val = strtod (str, &tail);
  if (errno == ERANGE && *tail == '\0') {
    g_error ("parse error: invalid double number \"%s\"", str);
  } else if (errno || tail == str || *tail != '\0') {
    g_error ("parse error: malformed double number \"%s\"", str);
  }
  return  val;
}

/**
 * snprint_strength:
 * @str: a buffer to print the rank into
 * @size: length of the buffer
 * @strength: the strength value to print
 *
 * Prints the rank, which is closest to strength into @str.
 * This is the inverse operation to rank_to_strength().
 */
void
snprint_strength (char *str, size_t size, double strength)
{
  int x = (int)(strength+10000.5) - 10000;

  if (x < 0) {
    snprintf (str, size, "%dk", -x);
  } else {
    snprintf (str, size, "%dd", x+1);
  }
}
