/* control.c - the GoList control center
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: control.c,v 1.5 2001/09/01 16:39:09 voss Exp $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <gnome.h>

#include <go-data.h>
#include <go-strengths.h>

#include "golist.h"

#include "players.xpm"
#include "games.xpm"
#include "machine.xpm"

GtkObject *strengths;
static  guint  idle_handle;
static  gboolean  idle_active = FALSE;


struct control_manager {
  GtkWidget *window, *appbar, *selector;
  GtkWidget *games, *players;
  GtkWidget *comp_button;
  char *filename;
  gboolean  changed;
};
static  struct control_manager *cm;


static void
update_statusbar (struct control_manager *cm)
{
  gchar *status;
  const char *name;

  if (cm->filename) {
    name = strrchr (cm->filename, '/');
    name = name ? name+1 : cm->filename;
  } else {
    name = "no name";
  }
  status = g_strdup_printf (_(" %s %s: %d players, %d games"),
			    cm->changed ? "**" : "--",
			    name,
			    go_data_player_count (GO_DATA (data)),
			    go_data_game_count (GO_DATA (data)));
  gnome_appbar_set_default (GNOME_APPBAR (cm->appbar), status);
  g_free (status);
}

/**********************************************************************
 * the animation
 */

#include "anim/frames.h"

static char ** image_data [] = {
  zz1_xpm,
  zz2_xpm,
  zz3_xpm,
  zz4_xpm,
  zz5_xpm,
  zz6_xpm,
  zz7_xpm,
  zz8_xpm,
  zz9_xpm,
  zz10_xpm,
  zz11_xpm,
  zz12_xpm,
  zz13_xpm,
  zz14_xpm,
  zz15_xpm,
  zz16_xpm,
  zz17_xpm,
  zz18_xpm,
  zz19_xpm,
  zz20_xpm
};
static int phase = 0;

static void
set_anim_image (struct control_manager *cm, int i)
{
  GtkWidget *vbox = GTK_BIN(cm->comp_button)->child;
  GtkBoxChild *x = GTK_BOX(vbox)->children->next->data;
  GnomePixmap *pixmap = GNOME_PIXMAP (x->widget);
  gnome_pixmap_load_xpm_d (pixmap, i<0 ? machine_xpm : image_data[i]);
}

/**********************************************************************
 * the estimation process
 */

static gint
estimator (gpointer user_data)
{
  struct control_manager *cm = user_data;

  set_anim_image (cm, phase);
  phase = (phase+19)%20;
  go_strengths_optimize (GO_STRENGTHS(strengths));
  return  TRUE;
}

/**********************************************************************
 * handle the user interface
 */

static gchar *
query_filename (struct control_manager *cm, const gchar *title,
		gboolean must_exist)
{
  GtkWidget *dialog, *fileentry, *label;
  int  reply;
  char *name = NULL;
  
  dialog = gnome_dialog_new (_("GoList: choose file"),
			     GNOME_STOCK_BUTTON_OK,
			     GNOME_STOCK_BUTTON_CANCEL,
			     NULL);
  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (cm->window));
  gnome_dialog_set_close (GNOME_DIALOG (dialog), TRUE);
  gnome_dialog_close_hides (GNOME_DIALOG (dialog), TRUE);

  label = gtk_label_new (title);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
		      label, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
  
  fileentry = gnome_file_entry_new ("golist:golist_loadsave_history",
				    _("GoList: Browse Files For Open"));
  gnome_dialog_editable_enters
    (GNOME_DIALOG (dialog),
     GTK_EDITABLE (gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(fileentry))));
  gnome_dialog_set_default (GNOME_DIALOG (dialog), GNOME_OK);
  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
		      fileentry, FALSE, FALSE, 0);

  gtk_widget_show_all (dialog);
  reply = gnome_dialog_run (GNOME_DIALOG (dialog));
  if (reply == GNOME_OK) {
    name = gnome_file_entry_get_full_path (GNOME_FILE_ENTRY(fileentry),
					   must_exist);
  }
  gtk_widget_destroy (dialog);
  return  name;
}

static gboolean
save_file (struct control_manager *cm, const char *filename)
{
  if (go_data_save (GO_DATA (data), filename)) {
    if (filename != cm->filename) {
      g_free (cm->filename);
      cm->filename = g_strdup (filename);
    }
    cm->changed = FALSE;
    return  TRUE;
  } else {
    GtkWidget *dialog;
    gchar *msg;

    msg = g_strdup_printf ("cannot write \"%s\"", filename);
    dialog = gnome_error_dialog_parented (msg, GTK_WINDOW (cm->window));
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gdk_beep ();
    g_free (msg);
    return  FALSE;
  }
}

static gboolean
load_file (struct control_manager *cm, const char *filename)
{
  if (go_data_load (GO_DATA (data), filename)) {
    g_free (cm->filename);
    cm->filename = g_strdup (filename);
    cm->changed = FALSE;
    return  TRUE;
  } else {
    GtkWidget *dialog;
    gchar *msg;

    msg = g_strdup_printf ("cannot read \"%s\"", filename);
    dialog = gnome_error_dialog_parented (msg, GTK_WINDOW (cm->window));
    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
    gdk_beep ();
    g_free (msg);
    return  FALSE;
  }
}

static gboolean
offer_save (struct control_manager *cm)
{
  GtkWidget *dialog;
  int  res;

  if (! cm->changed)  return  TRUE;
  
  dialog = gnome_message_box_new (_("The database is modified but not saved.\n"
				    "Save it now?"),
				  GNOME_MESSAGE_BOX_QUESTION,
				  "Save",
				  "Do not save",
				  GNOME_STOCK_BUTTON_CANCEL,
				  NULL);
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  res = gnome_dialog_run (GNOME_DIALOG (dialog));

  if (res == 2)  return  FALSE;
  if (res == 1) {
    cm->changed = FALSE;
    return  TRUE;
  }

  if (! cm->filename)  cm->filename = query_filename (cm,
						      _("Save database:"),
						      FALSE);
  if (! cm->filename)  return FALSE;

  return  save_file (cm, cm->filename);
}

static void
new_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct control_manager *cm = user_data;

  if (! offer_save (cm))  return;
  go_data_clear (GO_DATA (data));
  cm->changed = FALSE;
  update_statusbar (cm);
}

static void
open_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct control_manager *cm = user_data;
  gchar *filename;

  if (! offer_save (cm))  return;
  go_data_clear (GO_DATA (data));
  filename = query_filename (cm, _("Open database:"), TRUE);
  if (filename) {
    load_file (cm, filename);
    g_free (filename);
  }
  update_statusbar (cm);
}

static void
save_as_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct control_manager *cm = user_data;
  gchar *filename;

  filename = query_filename (cm, _("Save database:"), FALSE);
  if (filename) {
    save_file (cm, filename);
    g_free (filename);
  }
  update_statusbar (cm);
}

static void
save_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct control_manager *cm = user_data;
  if (cm->filename) {
    save_file (cm, cm->filename);
  } else {
    save_as_cb (menuitem, user_data);
  }
  update_statusbar (cm);
}

void
quit_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  if (! offer_save (cm))  return;
  gtk_main_quit ();
}

static void
about_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  about_window_open ();
}

static GnomeUIInfo file_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_NEW_ITEM (N_("New database"),
			     N_("Create a new empty database"),
			     new_cb, NULL),
  GNOMEUIINFO_MENU_OPEN_ITEM (open_cb, NULL),  
  GNOMEUIINFO_MENU_SAVE_ITEM (save_cb, NULL),  
  GNOMEUIINFO_MENU_SAVE_AS_ITEM (save_as_cb, NULL),  
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (quit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
  GNOMEUIINFO_END
};

static void
unselect_cb (GtkObject *obj, gpointer user_data)
{
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data), FALSE);
}

static void
players_cb (GtkToggleButton *togglebutton, gpointer user_data)
{
  struct control_manager *cm = user_data;

  if (gtk_toggle_button_get_active (togglebutton)) {
    cm->players = player_manager_open (data);
    gtk_signal_connect (GTK_OBJECT (cm->players), "destroy",
			GTK_SIGNAL_FUNC (unselect_cb), togglebutton);
  } else {
    gtk_widget_destroy (cm->players);
  }
}

static void
games_cb (GtkToggleButton *togglebutton, gpointer user_data)
{
  struct control_manager *cm = user_data;

  if (gtk_toggle_button_get_active (togglebutton)) {
    cm->games = game_manager_open (data);
    gtk_signal_connect (GTK_OBJECT (cm->games), "destroy",
			GTK_SIGNAL_FUNC (unselect_cb), togglebutton);
  } else {
    gtk_widget_destroy (cm->games);
  }
}

static void
compute_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct control_manager *cm = user_data;

  if (idle_active) {
    gtk_idle_remove (idle_handle);
    set_anim_image (cm, -1);
    idle_active = FALSE;
  } else {
    const char *error;

    error = go_strengths_check_preconditions (GO_STRENGTHS(strengths));
    if (error) {
      GtkWidget *dialog;
      dialog = gnome_error_dialog (error);
      gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
      gdk_beep ();
    } else {
      idle_handle = gtk_idle_add (estimator, cm);
      idle_active = TRUE;
    }
  }
  update_statusbar (cm);
}

static GnomeUIInfo toolbar_uiinfo[] =
{
  GNOMEUIINFO_TOGGLEITEM (N_("games"),
			  N_("open the game window"),
			  games_cb,
			  games_xpm),
  GNOMEUIINFO_TOGGLEITEM (N_("players"),
			  N_("open the player window"),
			  players_cb,
			  players_xpm),
  GNOMEUIINFO_TOGGLEITEM (N_("compute strengths"),
			  N_("toggle the estimation process"),
			  compute_cb,
			  machine_xpm),
  GNOMEUIINFO_END
};

static void
create_window (struct control_manager *cm)
{
  GtkWidget *window, *appbar;

  window = gnome_app_new ("GoList", _("GoList - control center"));
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (window),
			  "goListControlCenter", "GoList");

  appbar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
  gnome_app_set_statusbar (GNOME_APP(window), appbar);
  cm->appbar = appbar;
  
  gnome_app_create_menus_with_data (GNOME_APP (window), menubar_uiinfo, cm);
  gnome_app_install_menu_hints (GNOME_APP (window), menubar_uiinfo);
  gnome_app_create_toolbar_with_data (GNOME_APP (window), toolbar_uiinfo, cm);
  cm->comp_button = toolbar_uiinfo[2].widget;
  
  cm->window = window;
}

static gboolean
delete_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data)
{
  struct control_manager *cm = user_data;
  return  (! offer_save (cm));
}

static void
destroy_cb (GtkObject *obj, gpointer user_data)
{
  gtk_main_quit ();
}

void
data_changed_cb (GoData *data, gpointer user_data)
{
  struct control_manager *cm = user_data;
  cm->changed = TRUE;
  update_statusbar (cm);
}

void
control_manager_open (const char *filename)
{
  cm = g_new (struct control_manager, 1);
  cm->filename = NULL;
  cm->changed = FALSE;
  create_window (cm);
  cm->games = NULL;
  cm->players = NULL;

  ++main_loop_count;
  gtk_signal_connect (GTK_OBJECT (cm->window), "delete_event",
                      GTK_SIGNAL_FUNC (delete_cb), cm);
  gtk_signal_connect (GTK_OBJECT (cm->window), "destroy",
                      GTK_SIGNAL_FUNC (destroy_cb), cm);

  gtk_signal_connect (data, GO_DATA_SIGNAL_CHANGED (data_changed_cb), cm);
  if (filename)  load_file (cm, filename);
  update_statusbar (cm);
  
  strengths = go_strengths_new (GO_DATA (data), optimize_contants);

  gtk_widget_show_all (cm->window);
}
