/* games.c - implement the game-manager
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: games.c,v 1.4 2001/09/01 16:39:38 voss Exp $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include <go-game.h>
#include <go-game-list.h>

#include "golist.h"


struct game_manager {
  GtkWidget *window, *edit_entry;
  GtkCList *clist;
  GtkObject *game_list;
  int  select;

  /* dialog entries */
  GtkWidget *date, *black, *white, *board, *hc, *komi;
  GtkWidget *bbutton, *jbutton, *wbutton;

  /* edit game */
  GoGame *g;
};

/**********************************************************************
 * the add-games dialog
 */

static gchar *
option_menu_get_text (GtkWidget *option_menu)
{
  gchar *str;

  gtk_object_get (GTK_OBJECT (option_menu),
		  "label", &str,
		  NULL);
  return  str;
}

static void
construct_contents (struct game_manager *gm, GtkWidget *dialog)
{
  GoData *data = GO_CLIENT(gm->game_list)->data;
  GtkWidget *table, *label, *hbox, *menu;
  GtkObject *adj;
  GSList *winner_group = NULL;

  /* construct the dialog contents */
  table = gtk_table_new (4, 4, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);

  label = gtk_label_new (_("date: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("players: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("result: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  label = gtk_label_new (_("game: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  gm->date = gnome_date_edit_new ((time_t) 0, FALSE, TRUE);
  gtk_table_attach (GTK_TABLE (table), gm->date, 1, 4, 0, 1,
		    GTK_EXPAND|GTK_FILL, 0, 0, 0);
  gnome_date_edit_set_flags (GNOME_DATE_EDIT (gm->date),
			     GNOME_DATE_EDIT_WEEK_STARTS_ON_MONDAY);

  gm->black = gtk_combo_new ();
  gtk_table_attach (GTK_TABLE (table), gm->black, 1, 2, 1, 2,
                    GTK_EXPAND|GTK_FILL, 0, 0, 0);
  gtk_combo_set_popdown_strings (GTK_COMBO(gm->black),
				 go_data_player_tokens (data));
  gtk_combo_set_value_in_list (GTK_COMBO(gm->black), TRUE, FALSE);

  label = gtk_label_new (" - ");
  gtk_table_attach (GTK_TABLE (table), label, 2, 3, 1, 2,
                    0, GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  gm->white = gtk_combo_new ();
  gtk_table_attach (GTK_TABLE (table), gm->white, 3, 4, 1, 2,
                    GTK_EXPAND|GTK_FILL, 0, 0, 0);
  gtk_combo_set_popdown_strings (GTK_COMBO(gm->white),
				 go_data_player_tokens (data));
  gtk_combo_set_value_in_list (GTK_COMBO(gm->white), TRUE, FALSE);

  gm->bbutton = gtk_radio_button_new_with_label (winner_group,
						 _("black wins"));
  winner_group = gtk_radio_button_group (GTK_RADIO_BUTTON (gm->bbutton));
  gtk_table_attach (GTK_TABLE (table), gm->bbutton, 1, 2, 2, 3,
                    GTK_EXPAND, 0, 0, 0);

  gm->jbutton = gtk_radio_button_new_with_label (winner_group, _("jigo"));
  winner_group = gtk_radio_button_group (GTK_RADIO_BUTTON (gm->jbutton));
  gtk_table_attach (GTK_TABLE (table), gm->jbutton, 2, 3, 2, 3,
                    0, 0, 0, 0);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gm->jbutton), TRUE);

  gm->wbutton = gtk_radio_button_new_with_label (winner_group,
						 _("white wins"));
  winner_group = gtk_radio_button_group (GTK_RADIO_BUTTON (gm->wbutton));
  gtk_table_attach (GTK_TABLE (table), gm->wbutton, 3, 4, 2, 3,
                    0, 0, 0, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_table_attach (GTK_TABLE (table), hbox, 1, 4, 3, 4,
                    GTK_EXPAND|GTK_SHRINK|GTK_FILL, GTK_FILL, 0, 0);

  label = gtk_label_new (_("board: "));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  gm->board = gtk_option_menu_new ();
  menu = gtk_menu_new ();
  gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label ("19"));
  gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label ("13"));
  gtk_menu_append (GTK_MENU (menu), gtk_menu_item_new_with_label ("9"));
  gtk_option_menu_set_menu (GTK_OPTION_MENU (gm->board), menu);
  gtk_box_pack_start (GTK_BOX (hbox), gm->board, TRUE, TRUE, 0);

  label = gtk_label_new (_("  handicap: "));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);

  adj = gtk_adjustment_new (0, 0, 21, 1, 10, 10);
  gm->hc = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 0);
  gtk_box_pack_start (GTK_BOX (hbox), gm->hc, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (gm->hc), TRUE);

  label = gtk_label_new (_("  komi: "));
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  adj = gtk_adjustment_new (5, -99, 99, 0.5, 5, 5);
  gm->komi = gtk_spin_button_new (GTK_ADJUSTMENT (adj), 1, 1);
  gtk_box_pack_start (GTK_BOX (hbox), gm->komi, TRUE, TRUE, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (gm->komi), TRUE);

  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
		      table, TRUE, TRUE, 0);
}

static void
fill_contents (struct game_manager *gm, GoGame *g)
{
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(gm->black)->entry),
		      GO_PLAYER(g->black)->token);
  gtk_entry_set_text (GTK_ENTRY (GTK_COMBO(gm->white)->entry),
		      GO_PLAYER(g->white)->token);
  gnome_date_edit_set_time (GNOME_DATE_EDIT (gm->date), g->date);
  switch (g->res) {
  case res_BLACK:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gm->bbutton), TRUE);
    break;
  case res_WHITE:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gm->wbutton), TRUE);
    break;
  default:
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gm->jbutton), TRUE);
    break;
  }
  switch (g->board) {
  case b19:
    gtk_option_menu_set_history (GTK_OPTION_MENU (gm->board), 0);
    break;
  case b13:
    gtk_option_menu_set_history (GTK_OPTION_MENU (gm->board), 1);
    break;
  case b9:
    gtk_option_menu_set_history (GTK_OPTION_MENU (gm->board), 2);
    break;
  }
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (gm->hc), g->handicap);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON (gm->komi),
			     g->with_jigo ? g->komi : g->komi+0.5);
}

static void
ag_clicked_cb (GnomeDialog *dialog, gint button, gpointer user_data)
{
  struct game_manager *gm = user_data;
  time_t  date;
  gchar *black, *white, *board;
  int  hc;
  double  komi;
  GoGameResult  res;

  if (button != 0) {
    gnome_dialog_close (dialog);
    return;
  }

  black = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO(gm->black)->entry));
  white = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO(gm->white)->entry));
  date = gnome_date_edit_get_date (GNOME_DATE_EDIT (gm->date));
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(gm->bbutton))) {
    res = res_BLACK;
  } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(gm->wbutton))) {
    res = res_WHITE;
  } else {
    res = res_JIGO;
  }
  board = option_menu_get_text (gm->board);
  hc = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (gm->hc));
  komi = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (gm->komi));
  go_game_new (GO_CLIENT(gm->game_list)->data,
	       date, black, white, str_to_board (board), hc, komi, res);
  g_free (board);
}

static void
ag_dialog_open (struct game_manager *gm)
{
  GtkWidget *dialog;

  dialog = gnome_dialog_new (_("GoList - Add new games"),
			     _("Add"),
			     GNOME_STOCK_BUTTON_CLOSE,
			     NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (gm->window));
  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "goListAddGame", "GoList");

  construct_contents (gm, dialog);
  
  gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
		      GTK_SIGNAL_FUNC (ag_clicked_cb), gm);
  gtk_widget_show_all (dialog);
}

static void
eg_clicked_cb (GnomeDialog *dialog, gint button, gpointer user_data)
{
  struct game_manager *gm = user_data;
  time_t  date;
  gchar *black, *white, *board;
  int  hc;
  double  komi;
  GoGameResult  res;

  if (button != 0) {
    gnome_dialog_close (dialog);
    return;
  }

  go_data_game_lock (GO_CLIENT (gm->game_list)->data, gm->g);
  
  black = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO(gm->black)->entry));
  white = gtk_entry_get_text (GTK_ENTRY (GTK_COMBO(gm->white)->entry));
  date = gnome_date_edit_get_date (GNOME_DATE_EDIT (gm->date));
  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(gm->bbutton))) {
    res = res_BLACK;
  } else if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(gm->wbutton))) {
    res = res_WHITE;
  } else {
    res = res_JIGO;
  }
  board = option_menu_get_text (gm->board);
  hc = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (gm->hc));
  komi = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON (gm->komi));
  gtk_object_set (GTK_OBJECT (gm->g),
		  GO_GAME_ARG_BLACK (black),
		  GO_GAME_ARG_WHITE (white),
		  GO_GAME_ARG_BOARD (board),
		  GO_GAME_ARG_HANDICAP (hc),
		  GO_GAME_ARG_KOMI (komi),
		  NULL);
  gm->g->date = date;
  gm->g->res = res;
  gm->g->probability = -1;
  g_free (board);
  
  go_data_game_unlock (GO_CLIENT (gm->game_list)->data, gm->g);
  gnome_dialog_close (dialog);
}

static void
eg_dialog_open (struct game_manager *gm, GoGame *g)
{
  GtkWidget *dialog;

  dialog = gnome_dialog_new (_("GoList - Edit game"),
			     GNOME_STOCK_BUTTON_OK,
			     GNOME_STOCK_BUTTON_CANCEL,
			     NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (gm->window));
  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "goListEditGame", "GoList");
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  construct_contents (gm, dialog);
  fill_contents (gm, g);
  gm->g = g;
  
  gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
		      GTK_SIGNAL_FUNC (eg_clicked_cb), gm);
  gtk_widget_show_all (dialog);
}

/**********************************************************************
 * handle the user interface
 */

static void
print_gamelist_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct game_manager *gm = user_data;
  print_games (gm->game_list);
}

static void
close_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct game_manager *gm = user_data;
  gtk_widget_destroy (gm->window);
}

static void
add_games_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct game_manager *gm = user_data;
  ag_dialog_open (gm);
}

static void
edit_game_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct game_manager *gm = user_data;
  GoGame *game = go_client_game (GO_CLIENT (gm->game_list), gm->select);
  eg_dialog_open (gm, game);
}

static void
about_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  about_window_open ();
}

static GnomeUIInfo file_menu_uiinfo[] =
{
  GNOMEUIINFO_ITEM_NONE (N_("Print all games ..."),
			 N_("Print the full list of games"),
			 print_gamelist_cb),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_CLOSE_ITEM (close_cb, NULL),
  GNOMEUIINFO_MENU_EXIT_ITEM (quit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu_uiinfo[] =
{
  GNOMEUIINFO_ITEM_NONE (N_("Add new games ..."),
			 N_("Add new games to the database"),
			 add_games_cb),
  GNOMEUIINFO_ITEM_NONE (N_("Edit game ..."),
			 N_("Modify the database entry for the current game"),
			 edit_game_cb),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
  GNOMEUIINFO_MENU_EDIT_TREE (edit_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
  GNOMEUIINFO_END
};

static void
select_cb (GtkCList *clist, gint row, gint column, GdkEventButton *event,
	   gpointer user_data)
{
  struct game_manager *gm = user_data;
  gm->select = row;
  gtk_widget_set_sensitive (gm->edit_entry, TRUE);
}

static void
unselect_cb (GtkCList *clist, gint row, gint column, GdkEventButton *event,
	     gpointer user_data)
{
  struct game_manager *gm = user_data;
  gm->select = -1;
  gtk_widget_set_sensitive (gm->edit_entry, FALSE);
}

static void
create_window (struct game_manager *gm)
{
  GtkWidget *window, *sw, *gamelist;
  gchar *titles [] = {
    N_("date"),
    N_("black"),
    N_("white"),
    N_("board"),
    N_("handicap"),
    N_("komi"),
    N_("result"),
    N_("prob.")
  };
  
  window = gnome_app_new ("GoList", _("GoList - game manager"));
  gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (window),
			  "goListGameManager", "GoList");
  gtk_window_set_default_size (GTK_WINDOW (window),
			       0, gdk_screen_height()*0.65);
  
  gnome_app_create_menus_with_data (GNOME_APP (window), menubar_uiinfo, gm);
  gm->edit_entry = edit_menu_uiinfo[1].widget;
  gtk_widget_set_sensitive (gm->edit_entry, FALSE);

  gamelist = gtk_clist_new_with_titles (8, titles);
  gm->select = -1;
  gtk_signal_connect (GTK_OBJECT (gamelist), "select-row",
                      GTK_SIGNAL_FUNC (select_cb), gm);
  gtk_signal_connect (GTK_OBJECT (gamelist), "unselect-row",
                      GTK_SIGNAL_FUNC (unselect_cb), gm);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(sw),
				  GTK_POLICY_NEVER,
				  GTK_POLICY_ALWAYS);
  gtk_container_add (GTK_CONTAINER(sw), gamelist);
  
  gnome_app_set_contents (GNOME_APP (window), sw);

  gm->window = window;
  gm->clist = GTK_CLIST (gamelist);
}

static void
destroy_cb (GtkObject *obj, gpointer user_data)
{
  struct game_manager *gm = user_data;

  gtk_object_destroy (gm->game_list);
  g_free (gm);
  
  --main_loop_count;
  if (main_loop_count <= 0)  gtk_main_quit ();
}

GtkWidget *
game_manager_open (GtkObject *data)
{
  struct game_manager *gm;

  gm = g_new (struct game_manager, 1);
  create_window (gm);
  
  gm->game_list = go_game_list_new (GO_DATA (data), gm->clist);

  ++main_loop_count;
  gtk_signal_connect (GTK_OBJECT (gm->window), "destroy",
                      GTK_SIGNAL_FUNC (destroy_cb), gm);

  gtk_widget_show_all (gm->window);
  gtk_clist_columns_autosize (gm->clist);

  return  gm->window;
}
