/* players.c - implement the player-manager
 *
 * Copyright (C) 2001  Jochen Voss.  */

static const  char  rcsid[] = "$Id: players.c,v 1.4 2001/09/01 16:39:44 voss Exp $";

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <string.h>
#include <gnome.h>

#include <go-player.h>
#include <go-player-list.h>
#include <go-components.h>

#include "golist.h"


struct player_manager {
  GtkWidget *window, *sheet_entry, *edit_entry;
  GtkCList *clist;
  GtkObject *player_list;
  int  select;

  /* dialog entries */
  GtkWidget *nentry, *tentry, *rentry;

  /* edit player */
  GoPlayer *p;
};

/**********************************************************************
 * the add-players dialog
 */

static void
complain_rank (struct player_manager *pm, const char *rank)
{
  GtkWidget *dialog;
  gchar *msg;
  
  msg = g_strdup_printf ("Invalid rank \"%s\"!\n"
			 "The rank should be of the form\n"
			 "12k (for 12 kyu) or 3d (for 3 dan)\n"
			 "If the rank is unknown leave this field empty.",
			 rank);
  dialog = gnome_error_dialog_parented (msg, GTK_WINDOW (pm->window));
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gdk_beep ();
  g_free (msg);
}

static void
complain_token (struct player_manager *pm, const char *token)
{
  GtkWidget *dialog;
  gchar *msg;
  
  msg = g_strdup_printf ("Invalid token \"%s\"!\n"
			 "The token should be a short string\n"
			 "consisting only of alphanumeric characters.\n"
			 "It must identify the player uniquely\n"
			 "and must not be empty.",
			 token);
  dialog = gnome_error_dialog_parented (msg, GTK_WINDOW (pm->window));
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
  gdk_beep ();
  g_free (msg);
}

static void
construct_contents (struct player_manager *pm, GtkWidget *dialog)
{
  GtkWidget *table, *label, *entry;

  /* construct the dialog contents */
  table = gtk_table_new (3, 2, FALSE);
  gtk_container_set_border_width (GTK_CONTAINER (table), 5);
  gtk_table_set_row_spacings (GTK_TABLE (table), 2);

  label = gtk_label_new (_("name: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 0, 1,
		    GTK_FILL|GTK_EXPAND, 0, 0, 0);
  pm->nentry = entry;
  
  label = gtk_label_new (_("token: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 1, 2,
		    GTK_FILL|GTK_EXPAND, 0, 0, 0);
  pm->tentry = entry;

  label = gtk_label_new (_("rank: "));
  gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

  entry = gtk_entry_new ();
  gtk_table_attach (GTK_TABLE (table), entry, 1, 2, 2, 3,
		    GTK_FILL|GTK_EXPAND, 0, 0, 0);
  pm->rentry = entry;

  gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox),
		      table, TRUE, TRUE, 0);
}

static void
fill_contents (struct player_manager *pm, GoPlayer *p)
{
  gtk_entry_set_text (GTK_ENTRY (pm->tentry), p->token);
  gtk_widget_set_sensitive (pm->tentry, FALSE);
  gtk_entry_set_text (GTK_ENTRY (pm->nentry), p->name ? p->name : "");
  gtk_entry_set_text (GTK_ENTRY (pm->rentry), p->rank ? p->rank : "");
}

static void
ap_clicked_cb (GnomeDialog *dialog, gint button, gpointer user_data)
{
  struct player_manager *pm = user_data;
  gchar *name, *token, *rank;
  double  strength;

  if (button != 0) {
    gnome_dialog_close (dialog);
    return;
  }

  name = gtk_entry_get_text (GTK_ENTRY (pm->nentry));
  token = gtk_entry_get_text (GTK_ENTRY (pm->tentry));
  if (! token || ! *token) {
    complain_token (pm, token);
    return;
  }
  rank = gtk_entry_get_text (GTK_ENTRY (pm->rentry));
  if (! *rank)  rank = NULL;
  if (! rank_to_strength (rank, &strength)) {
    complain_rank (pm, rank);
    return;
  }
  go_player_new (GO_CLIENT(pm->player_list)->data,
		 name, token, rank, strength);

  gtk_entry_set_text (GTK_ENTRY (pm->nentry), "");
  gtk_entry_set_text (GTK_ENTRY (pm->tentry), "");
  gtk_entry_set_text (GTK_ENTRY (pm->rentry), "");
}

static void
ap_dialog_open (struct player_manager *pm)
{
  GtkWidget *dialog;

  dialog = gnome_dialog_new (_("GoList - Add new players"),
			     _("Add"),
			     GNOME_STOCK_BUTTON_CLOSE,
			     NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (pm->window));
  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "goListAddPlayer", "GoList");

  construct_contents (pm, dialog);
  
  gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
		      GTK_SIGNAL_FUNC (ap_clicked_cb), pm);
  gtk_widget_show_all (dialog);
}

static void
ep_clicked_cb (GnomeDialog *dialog, gint button, gpointer user_data)
{
  struct player_manager *pm = user_data;
  gchar *name, *rank;

  if (button != 0) {
    gnome_dialog_close (dialog);
    return;
  }

  go_data_player_lock (GO_CLIENT (pm->player_list)->data, pm->p);

  name = gtk_entry_get_text (GTK_ENTRY (pm->nentry));
  rank = gtk_entry_get_text (GTK_ENTRY (pm->rentry));
  if (! *rank)  rank = NULL;
  if (! rank_to_strength (rank, &pm->p->strength)) {
    complain_rank (pm, rank);
    return;
  }
  gtk_object_set (GTK_OBJECT (pm->p),
		  GO_PLAYER_ARG_NAME (name),
		  GO_PLAYER_ARG_RANK (rank),
		  NULL);
  go_data_notify_players_changed (GO_CLIENT (pm->player_list)->data);

  go_data_player_unlock (GO_CLIENT (pm->player_list)->data, pm->p);
  gnome_dialog_close (dialog);
}

static void
ep_dialog_open (struct player_manager *pm, GoPlayer *p)
{
  GtkWidget *dialog;

  dialog = gnome_dialog_new (_("GoList - Edit player"),
			     GNOME_STOCK_BUTTON_OK,
			     GNOME_STOCK_BUTTON_CANCEL,
			     NULL);
  gnome_dialog_set_parent (GNOME_DIALOG (dialog), GTK_WINDOW (pm->window));
  gtk_window_set_policy (GTK_WINDOW (dialog), FALSE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (dialog), "goListEditPlayer", "GoList");
  gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);

  construct_contents (pm, dialog);
  fill_contents (pm, p);
  pm->p = p;

  gtk_signal_connect (GTK_OBJECT (dialog), "clicked",
		      GTK_SIGNAL_FUNC (ep_clicked_cb), pm);
  gtk_widget_show_all (dialog);
}

/**********************************************************************
 * handle the user interface
 */

static void
print_playerlist_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct player_manager *pm = user_data;
  print_players (pm->player_list);
}

static void
print_playersheet_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct player_manager *pm = user_data;
  
  print_sheet (pm->player_list,
	       go_client_player (GO_CLIENT (pm->player_list), pm->select));
}

static void
close_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct player_manager *pm = user_data;
  gtk_widget_destroy (pm->window);
}

static void
add_players_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct player_manager *pm = user_data;
  ap_dialog_open (pm);
}

static void
edit_player_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  struct player_manager *pm = user_data;
  GoPlayer *player = go_client_player (GO_CLIENT (pm->player_list),
				       pm->select);
  ep_dialog_open (pm, player);
}

static void
about_cb (GtkMenuItem *menuitem, gpointer user_data)
{
  about_window_open ();
}

static GnomeUIInfo file_menu_uiinfo[] =
{
  GNOMEUIINFO_ITEM_NONE (N_("Print player list ..."),
			 N_("Print the list of all players"),
			 print_playerlist_cb),
  GNOMEUIINFO_ITEM_NONE (N_("Print player sheet ..."),
			 N_("Print an info sheet for the selected player"),
			 print_playersheet_cb),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_CLOSE_ITEM (close_cb, NULL),
  GNOMEUIINFO_MENU_EXIT_ITEM (quit_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu_uiinfo[] =
{
  GNOMEUIINFO_ITEM_NONE (N_("Add new players ..."),
			 N_("Add new players to the database"),
			 add_players_cb),
  GNOMEUIINFO_ITEM_NONE (N_("Edit player ..."),
			 N_("Modify the database entry "
			    "for the current player"),
			 edit_player_cb),
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] =
{
  GNOMEUIINFO_MENU_ABOUT_ITEM (about_cb, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] =
{
  GNOMEUIINFO_MENU_FILE_TREE (file_menu_uiinfo),
  GNOMEUIINFO_MENU_EDIT_TREE (edit_menu_uiinfo),
  GNOMEUIINFO_MENU_HELP_TREE (help_menu_uiinfo),
  GNOMEUIINFO_END
};

static void
select_cb (GtkCList *clist, gint row, gint column, GdkEventButton *event,
	   gpointer user_data)
{
  struct player_manager *pm = user_data;
  pm->select = row;
  gtk_widget_set_sensitive (pm->sheet_entry, TRUE);
  gtk_widget_set_sensitive (pm->edit_entry, TRUE);
}

static void
unselect_cb (GtkCList *clist, gint row, gint column, GdkEventButton *event,
	     gpointer user_data)
{
  struct player_manager *pm = user_data;
  pm->select = -1;
  gtk_widget_set_sensitive (pm->sheet_entry, FALSE);
  gtk_widget_set_sensitive (pm->edit_entry, FALSE);
}

static void
create_window (struct player_manager *pm)
{
  GtkWidget *window, *sw, *playerlist;
  gchar *titles [] = {
    N_("name"),
    N_("token"),
    N_("rank"),
    N_("games"),
    N_("wins"),
    N_("c.rank"),
    N_("score")
  };
  
  window = gnome_app_new ("GoList", _("GoList - player manager"));
  gtk_window_set_policy (GTK_WINDOW (window), TRUE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (window),
			  "goListPlayerManager", "GoList");
  gtk_window_set_default_size (GTK_WINDOW (window),
			       0, gdk_screen_height()*0.45);
  
  gnome_app_create_menus_with_data (GNOME_APP (window), menubar_uiinfo, pm);
  pm->sheet_entry = file_menu_uiinfo[1].widget;
  pm->edit_entry = edit_menu_uiinfo[1].widget;
  gtk_widget_set_sensitive (pm->sheet_entry, FALSE);
  gtk_widget_set_sensitive (pm->edit_entry, FALSE);

  playerlist = gtk_clist_new_with_titles (7, titles);
  pm->select = -1;
  gtk_signal_connect (GTK_OBJECT (playerlist), "select-row",
                      GTK_SIGNAL_FUNC (select_cb), pm);
  gtk_signal_connect (GTK_OBJECT (playerlist), "unselect-row",
                      GTK_SIGNAL_FUNC (unselect_cb), pm);

  sw = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(sw),
				  GTK_POLICY_NEVER,
				  GTK_POLICY_ALWAYS);
  gtk_container_add (GTK_CONTAINER(sw), playerlist);
  
  gnome_app_set_contents (GNOME_APP (window), sw);

  pm->window = window;
  pm->clist = GTK_CLIST (playerlist);
}

static int
compare_players (const GoPlayer *a, const GoPlayer *b, void *client_data)
{
  int  ca, cb;

  ca = go_data_player_component (a->data, a);
  cb = go_data_player_component (a->data, b);
  if (ca == cb) {
    if (a->strength < b->strength)  return +1;
    if (a->strength > b->strength)  return -1;
    return  strcmp (a->token, b->token);
  }

  if (ca == 0)  return -1;
  if (cb == 0)  return +1;
  if (go_components_geq (GO_COMPONENTS(a->data->components), a, b))
    return -1;
  if (go_components_geq (GO_COMPONENTS(a->data->components), b, a))
    return +1;
  return  strcmp (a->token, b->token);
}

static void
destroy_cb (GtkObject *obj, gpointer user_data)
{
  struct player_manager *pm = user_data;

  gtk_object_destroy (pm->player_list);
  g_free (pm);
  
  --main_loop_count;
  if (main_loop_count <= 0)  gtk_main_quit ();
}

GtkWidget *
player_manager_open (GtkObject *data)
{
  struct player_manager *pm;

  pm = g_new (struct player_manager, 1);
  create_window (pm);
  
  pm->player_list = go_player_list_new (GO_DATA (data), pm->clist);
  go_client_sort_players (GO_CLIENT (pm->player_list), compare_players, NULL);

  ++main_loop_count;
  gtk_signal_connect (GTK_OBJECT (pm->window), "destroy",
                      GTK_SIGNAL_FUNC (destroy_cb), pm);

  gtk_widget_show_all (pm->window);
  gtk_clist_columns_autosize (pm->clist);

  return  pm->window;
}
